from selenium.webdriver.common.by import By

from nbformat.v4 import new_markdown_cell


def get_rendered_contents(nb):
    cl = ["text_cell", "render"]
    rendered_cells = [cell.find_element(By.CLASS_NAME, "text_cell_render")
                      for cell in nb.cells 
                      if all([c in cell.get_attribute("class") for c in cl])]
    return [x.get_attribute('innerHTML').strip()
            for x in rendered_cells 
            if x is not None]

            
def test_markdown_cell(prefill_notebook):
    nb = prefill_notebook([new_markdown_cell(md) for md in [
        '# Foo', '**Bar**', '*Baz*', '```\nx = 1\n```', '```aaaa\nx = 1\n```',
        '```python\ns = "$"\nt = "$"\n```'
    ]])

    assert get_rendered_contents(nb) == [
        '<h1 id="Foo">Foo<a class="anchor-link" href="#Foo">¶</a></h1>',
        '<p><strong>Bar</strong></p>',
        '<p><em>Baz</em></p>',
        '<pre><code>x = 1</code></pre>',
        '<pre><code class="cm-s-ipython language-aaaa">x = 1</code></pre>',
        '<pre><code class="cm-s-ipython language-python">' + 
        '<span class="cm-variable">s</span> <span class="cm-operator">=</span> <span class="cm-string">"$"</span>\n' +
        '<span class="cm-variable">t</span> <span class="cm-operator">=</span> <span class="cm-string">"$"</span></code></pre>'
    ]

def test_markdown_headings(notebook):
    lst = list([1, 2, 3, 4, 5, 6, 2, 1])
    for i in lst:
        notebook.add_markdown_cell()
        cell_text = notebook.browser.execute_script(f"""
            var cell = IPython.notebook.get_cell(1);
            cell.set_heading_level({i});
            cell.get_text();
        """)
        assert notebook.get_cell_contents(1) == "#" * i + " "
        notebook.delete_cell(1)




def cell_is_deletable(nb, index):
    JS = f'return Jupyter.notebook.get_cell({index}).is_deletable();'
    return nb.browser.execute_script(JS)

def remove_all_cells(notebook):
    for i in range(len(notebook.cells)):
        notebook.delete_cell(0)

INITIAL_CELLS = ['print("a")', 'print("b")', 'print("c")']

def test_delete_cells(prefill_notebook):
    a, b, c = INITIAL_CELLS
    notebook = prefill_notebook(INITIAL_CELLS)

    # Validate initial state
    assert notebook.get_cells_contents() == [a, b, c]
    for cell in range(0, 3):
        assert cell_is_deletable(notebook, cell)

    notebook.set_cell_metadata(0, 'deletable', 'false')
    notebook.set_cell_metadata(1, 'deletable', 0
    )
    assert not cell_is_deletable(notebook, 0)
    assert cell_is_deletable(notebook, 1)
    assert cell_is_deletable(notebook, 2)

    # Try to delete cell a (should not be deleted)
    notebook.delete_cell(0)
    assert notebook.get_cells_contents() == [a, b, c]

    # Try to delete cell b (should succeed)
    notebook.delete_cell(1)
    assert notebook.get_cells_contents() == [a, c]

    # Try to delete cell c (should succeed)
    notebook.delete_cell(1)
    assert notebook.get_cells_contents() == [a]

    # Change the deletable state of cell a
    notebook.set_cell_metadata(0, 'deletable', 'true')

    # Try to delete cell a (should succeed)
    notebook.delete_cell(0)
    assert len(notebook.cells) == 1 # it contains an empty cell

    # Make sure copied cells are deletable
    notebook.edit_cell(index=0, content=a)
    notebook.set_cell_metadata(0, 'deletable', 'false')
    assert not cell_is_deletable(notebook, 0)
    notebook.to_command_mode()
    notebook.current_cell.send_keys('cv')
    assert len(notebook.cells) == 2
    assert cell_is_deletable(notebook, 1)

    notebook.set_cell_metadata(0, 'deletable', 'true')  # to perform below test, remove all the cells
    remove_all_cells(notebook)
    assert len(notebook.cells) == 1    # notebook should create one automatically on empty notebook
